from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define, field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ServiceOncallUserSchema")


@_attrs_define
class ServiceOncallUserSchema:
    """
    Attributes:
        email (str):
        id (Union[None, Unset, int]):
        first_name (Union[None, Unset, str]):
        last_name (Union[None, Unset, str]):
    """

    email: str
    id: Union[None, Unset, int] = UNSET
    first_name: Union[None, Unset, str] = UNSET
    last_name: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        email = self.email

        id: Union[None, Unset, int]
        if isinstance(self.id, Unset):
            id = UNSET
        else:
            id = self.id

        first_name: Union[None, Unset, str]
        if isinstance(self.first_name, Unset):
            first_name = UNSET
        else:
            first_name = self.first_name

        last_name: Union[None, Unset, str]
        if isinstance(self.last_name, Unset):
            last_name = UNSET
        else:
            last_name = self.last_name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "email": email,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if first_name is not UNSET:
            field_dict["first_name"] = first_name
        if last_name is not UNSET:
            field_dict["last_name"] = last_name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        email = d.pop("email")

        def _parse_id(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        id = _parse_id(d.pop("id", UNSET))

        def _parse_first_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        first_name = _parse_first_name(d.pop("first_name", UNSET))

        def _parse_last_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        last_name = _parse_last_name(d.pop("last_name", UNSET))

        service_oncall_user_schema = cls(
            email=email,
            id=id,
            first_name=first_name,
            last_name=last_name,
        )

        service_oncall_user_schema.additional_properties = d
        return service_oncall_user_schema

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define, field as _attrs_field
from dateutil.parser import isoparse

T = TypeVar("T", bound="ScheduleOverrideInputSchema")


@_attrs_define
class ScheduleOverrideInputSchema:
    """
    Attributes:
        user_id (int):
        start (datetime.datetime):
        end (datetime.datetime):
    """

    user_id: int
    start: datetime.datetime
    end: datetime.datetime
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user_id = self.user_id

        start = self.start.isoformat()

        end = self.end.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "user_id": user_id,
                "start": start,
                "end": end,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        user_id = d.pop("user_id")

        start = isoparse(d.pop("start"))

        end = isoparse(d.pop("end"))

        schedule_override_input_schema = cls(
            user_id=user_id,
            start=start,
            end=end,
        )

        schedule_override_input_schema.additional_properties = d
        return schedule_override_input_schema

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define, field as _attrs_field
from dateutil.parser import isoparse

from ..models.incident_group_status import IncidentGroupStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.incident_group_service_schema import IncidentGroupServiceSchema


T = TypeVar("T", bound="IncidentGroupSchema")


@_attrs_define
class IncidentGroupSchema:
    """
    Attributes:
        status (IncidentGroupStatus):
        service (Union['IncidentGroupServiceSchema', None]):
        created_at (datetime.datetime):
        id (Union[None, Unset, int]):
        incident_number (Union[Unset, int]):  Default: 0.
        summary (Union[None, Unset, str]):
        acknowledged_at (Union[None, Unset, datetime.datetime]):
        resolved_at (Union[None, Unset, datetime.datetime]):
        silenced_at (Union[None, Unset, datetime.datetime]):
        last_status_change_at (Union[None, Unset, datetime.datetime]):
    """

    status: IncidentGroupStatus
    service: Union["IncidentGroupServiceSchema", None]
    created_at: datetime.datetime
    id: Union[None, Unset, int] = UNSET
    incident_number: Union[Unset, int] = 0
    summary: Union[None, Unset, str] = UNSET
    acknowledged_at: Union[None, Unset, datetime.datetime] = UNSET
    resolved_at: Union[None, Unset, datetime.datetime] = UNSET
    silenced_at: Union[None, Unset, datetime.datetime] = UNSET
    last_status_change_at: Union[None, Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.incident_group_service_schema import IncidentGroupServiceSchema

        status = self.status.value

        service: Union[Dict[str, Any], None]
        if isinstance(self.service, IncidentGroupServiceSchema):
            service = self.service.to_dict()
        else:
            service = self.service

        created_at = self.created_at.isoformat()

        id: Union[None, Unset, int]
        if isinstance(self.id, Unset):
            id = UNSET
        else:
            id = self.id

        incident_number = self.incident_number

        summary: Union[None, Unset, str]
        if isinstance(self.summary, Unset):
            summary = UNSET
        else:
            summary = self.summary

        acknowledged_at: Union[None, Unset, str]
        if isinstance(self.acknowledged_at, Unset):
            acknowledged_at = UNSET
        elif isinstance(self.acknowledged_at, datetime.datetime):
            acknowledged_at = self.acknowledged_at.isoformat()
        else:
            acknowledged_at = self.acknowledged_at

        resolved_at: Union[None, Unset, str]
        if isinstance(self.resolved_at, Unset):
            resolved_at = UNSET
        elif isinstance(self.resolved_at, datetime.datetime):
            resolved_at = self.resolved_at.isoformat()
        else:
            resolved_at = self.resolved_at

        silenced_at: Union[None, Unset, str]
        if isinstance(self.silenced_at, Unset):
            silenced_at = UNSET
        elif isinstance(self.silenced_at, datetime.datetime):
            silenced_at = self.silenced_at.isoformat()
        else:
            silenced_at = self.silenced_at

        last_status_change_at: Union[None, Unset, str]
        if isinstance(self.last_status_change_at, Unset):
            last_status_change_at = UNSET
        elif isinstance(self.last_status_change_at, datetime.datetime):
            last_status_change_at = self.last_status_change_at.isoformat()
        else:
            last_status_change_at = self.last_status_change_at

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "status": status,
                "service": service,
                "created_at": created_at,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if incident_number is not UNSET:
            field_dict["incident_number"] = incident_number
        if summary is not UNSET:
            field_dict["summary"] = summary
        if acknowledged_at is not UNSET:
            field_dict["acknowledged_at"] = acknowledged_at
        if resolved_at is not UNSET:
            field_dict["resolved_at"] = resolved_at
        if silenced_at is not UNSET:
            field_dict["silenced_at"] = silenced_at
        if last_status_change_at is not UNSET:
            field_dict["last_status_change_at"] = last_status_change_at

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.incident_group_service_schema import IncidentGroupServiceSchema

        d = src_dict.copy()
        status = IncidentGroupStatus(d.pop("status"))

        def _parse_service(data: object) -> Union["IncidentGroupServiceSchema", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                service_type_0 = IncidentGroupServiceSchema.from_dict(data)

                return service_type_0
            except:  # noqa: E722
                pass
            return cast(Union["IncidentGroupServiceSchema", None], data)

        service = _parse_service(d.pop("service"))

        created_at = isoparse(d.pop("created_at"))

        def _parse_id(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        id = _parse_id(d.pop("id", UNSET))

        incident_number = d.pop("incident_number", UNSET)

        def _parse_summary(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        summary = _parse_summary(d.pop("summary", UNSET))

        def _parse_acknowledged_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                acknowledged_at_type_0 = isoparse(data)

                return acknowledged_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        acknowledged_at = _parse_acknowledged_at(d.pop("acknowledged_at", UNSET))

        def _parse_resolved_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                resolved_at_type_0 = isoparse(data)

                return resolved_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        resolved_at = _parse_resolved_at(d.pop("resolved_at", UNSET))

        def _parse_silenced_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                silenced_at_type_0 = isoparse(data)

                return silenced_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        silenced_at = _parse_silenced_at(d.pop("silenced_at", UNSET))

        def _parse_last_status_change_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_status_change_at_type_0 = isoparse(data)

                return last_status_change_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        last_status_change_at = _parse_last_status_change_at(d.pop("last_status_change_at", UNSET))

        incident_group_schema = cls(
            status=status,
            service=service,
            created_at=created_at,
            id=id,
            incident_number=incident_number,
            summary=summary,
            acknowledged_at=acknowledged_at,
            resolved_at=resolved_at,
            silenced_at=silenced_at,
            last_status_change_at=last_status_change_at,
        )

        incident_group_schema.additional_properties = d
        return incident_group_schema

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define, field as _attrs_field

T = TypeVar("T", bound="UpdateHeartbeatInput")


@_attrs_define
class UpdateHeartbeatInput:
    """
    Attributes:
        name (str):
        description (str):
        link (Union[None, str]):
        service (int):
    """

    name: str
    description: str
    link: Union[None, str]
    service: int
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        description = self.description

        link: Union[None, str]
        link = self.link

        service = self.service

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "description": description,
                "link": link,
                "service": service,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        description = d.pop("description")

        def _parse_link(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        link = _parse_link(d.pop("link"))

        service = d.pop("service")

        update_heartbeat_input = cls(
            name=name,
            description=description,
            link=link,
            service=service,
        )

        update_heartbeat_input.additional_properties = d
        return update_heartbeat_input

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define, field as _attrs_field

from ..models.heartbeat_type import HeartbeatType

T = TypeVar("T", bound="UpdateHeartbeatStateInput")


@_attrs_define
class UpdateHeartbeatStateInput:
    """
    Attributes:
        type (HeartbeatType):
        cron_expression (Union[None, str]):
        cron_timezone (Union[None, str]):
        interval_seconds (Union[None, int]):
        timeout_seconds (int):
        muted (bool):
        resolve_incident (bool):
    """

    type: HeartbeatType
    cron_expression: Union[None, str]
    cron_timezone: Union[None, str]
    interval_seconds: Union[None, int]
    timeout_seconds: int
    muted: bool
    resolve_incident: bool
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        cron_expression: Union[None, str]
        cron_expression = self.cron_expression

        cron_timezone: Union[None, str]
        cron_timezone = self.cron_timezone

        interval_seconds: Union[None, int]
        interval_seconds = self.interval_seconds

        timeout_seconds = self.timeout_seconds

        muted = self.muted

        resolve_incident = self.resolve_incident

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "cron_expression": cron_expression,
                "cron_timezone": cron_timezone,
                "interval_seconds": interval_seconds,
                "timeout_seconds": timeout_seconds,
                "muted": muted,
                "resolve_incident": resolve_incident,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = HeartbeatType(d.pop("type"))

        def _parse_cron_expression(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        cron_expression = _parse_cron_expression(d.pop("cron_expression"))

        def _parse_cron_timezone(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        cron_timezone = _parse_cron_timezone(d.pop("cron_timezone"))

        def _parse_interval_seconds(data: object) -> Union[None, int]:
            if data is None:
                return data
            return cast(Union[None, int], data)

        interval_seconds = _parse_interval_seconds(d.pop("interval_seconds"))

        timeout_seconds = d.pop("timeout_seconds")

        muted = d.pop("muted")

        resolve_incident = d.pop("resolve_incident")

        update_heartbeat_state_input = cls(
            type=type,
            cron_expression=cron_expression,
            cron_timezone=cron_timezone,
            interval_seconds=interval_seconds,
            timeout_seconds=timeout_seconds,
            muted=muted,
            resolve_incident=resolve_incident,
        )

        update_heartbeat_state_input.additional_properties = d
        return update_heartbeat_state_input

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define, field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ServiceSchema")


@_attrs_define
class ServiceSchema:
    """
    Attributes:
        name (str):
        id (Union[None, Unset, int]):
        description (Union[None, Unset, str]):
        service_type (Union[Unset, str]):  Default: 'technical'.
        routing_key (Union[Unset, str]):
    """

    name: str
    id: Union[None, Unset, int] = UNSET
    description: Union[None, Unset, str] = UNSET
    service_type: Union[Unset, str] = "technical"
    routing_key: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        id: Union[None, Unset, int]
        if isinstance(self.id, Unset):
            id = UNSET
        else:
            id = self.id

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        service_type = self.service_type

        routing_key = self.routing_key

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if description is not UNSET:
            field_dict["description"] = description
        if service_type is not UNSET:
            field_dict["service_type"] = service_type
        if routing_key is not UNSET:
            field_dict["routing_key"] = routing_key

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        def _parse_id(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        id = _parse_id(d.pop("id", UNSET))

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))

        service_type = d.pop("service_type", UNSET)

        routing_key = d.pop("routing_key", UNSET)

        service_schema = cls(
            name=name,
            id=id,
            description=description,
            service_type=service_type,
            routing_key=routing_key,
        )

        service_schema.additional_properties = d
        return service_schema

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define, field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.cron_heartbeat_state_schema import CronHeartbeatStateSchema
    from ..models.interval_heartbeat_state_schema import IntervalHeartbeatStateSchema


T = TypeVar("T", bound="HeartbeatSchema")


@_attrs_define
class HeartbeatSchema:
    """
    Attributes:
        states (List[Union['CronHeartbeatStateSchema', 'IntervalHeartbeatStateSchema']]):
        name (str):
        service (int):
        id (Union[None, Unset, int]):
        public_primary_key (Union[Unset, str]):
        description (Union[None, Unset, str]):  Default: ''.
        link (Union[None, Unset, str]):
    """

    states: List[Union["CronHeartbeatStateSchema", "IntervalHeartbeatStateSchema"]]
    name: str
    service: int
    id: Union[None, Unset, int] = UNSET
    public_primary_key: Union[Unset, str] = UNSET
    description: Union[None, Unset, str] = ""
    link: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.cron_heartbeat_state_schema import CronHeartbeatStateSchema

        states = []
        for states_item_data in self.states:
            states_item: Dict[str, Any]
            if isinstance(states_item_data, CronHeartbeatStateSchema):
                states_item = states_item_data.to_dict()
            else:
                states_item = states_item_data.to_dict()

            states.append(states_item)

        name = self.name

        service = self.service

        id: Union[None, Unset, int]
        if isinstance(self.id, Unset):
            id = UNSET
        else:
            id = self.id

        public_primary_key = self.public_primary_key

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        link: Union[None, Unset, str]
        if isinstance(self.link, Unset):
            link = UNSET
        else:
            link = self.link

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "states": states,
                "name": name,
                "service": service,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if public_primary_key is not UNSET:
            field_dict["public_primary_key"] = public_primary_key
        if description is not UNSET:
            field_dict["description"] = description
        if link is not UNSET:
            field_dict["link"] = link

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.cron_heartbeat_state_schema import CronHeartbeatStateSchema
        from ..models.interval_heartbeat_state_schema import IntervalHeartbeatStateSchema

        d = src_dict.copy()
        states = []
        _states = d.pop("states")
        for states_item_data in _states:

            def _parse_states_item(data: object) -> Union["CronHeartbeatStateSchema", "IntervalHeartbeatStateSchema"]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    states_item_type_0 = CronHeartbeatStateSchema.from_dict(data)

                    return states_item_type_0
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                states_item_type_1 = IntervalHeartbeatStateSchema.from_dict(data)

                return states_item_type_1

            states_item = _parse_states_item(states_item_data)

            states.append(states_item)

        name = d.pop("name")

        service = d.pop("service")

        def _parse_id(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        id = _parse_id(d.pop("id", UNSET))

        public_primary_key = d.pop("public_primary_key", UNSET)

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_link(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        link = _parse_link(d.pop("link", UNSET))

        heartbeat_schema = cls(
            states=states,
            name=name,
            service=service,
            id=id,
            public_primary_key=public_primary_key,
            description=description,
            link=link,
        )

        heartbeat_schema.additional_properties = d
        return heartbeat_schema

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .create_heartbeat_input import CreateHeartbeatInput
from .create_heartbeat_state_input import CreateHeartbeatStateInput
from .cron_heartbeat_state_schema import CronHeartbeatStateSchema
from .cron_heartbeat_state_schema_type import CronHeartbeatStateSchemaType
from .heart_beat_filter import HeartBeatFilter
from .heartbeat_schema import HeartbeatSchema
from .heartbeat_type import HeartbeatType
from .incident_group_detail_note_schema import IncidentGroupDetailNoteSchema
from .incident_group_detail_schema import IncidentGroupDetailSchema
from .incident_group_filter import IncidentGroupFilter
from .incident_group_schema import IncidentGroupSchema
from .incident_group_service_schema import IncidentGroupServiceSchema
from .incident_group_status import IncidentGroupStatus
from .input_ import Input
from .interval_heartbeat_state_schema import IntervalHeartbeatStateSchema
from .interval_heartbeat_state_schema_type import IntervalHeartbeatStateSchemaType
from .page_info import PageInfo
from .paged_heartbeat_schema import PagedHeartbeatSchema
from .paged_incident_group_schema import PagedIncidentGroupSchema
from .paged_schedule_override_schema import PagedScheduleOverrideSchema
from .paged_schedule_schema import PagedScheduleSchema
from .paged_service_schema import PagedServiceSchema
from .paged_user_schema import PagedUserSchema
from .schedule_override_filter import ScheduleOverrideFilter
from .schedule_override_input_schema import ScheduleOverrideInputSchema
from .schedule_override_schema import ScheduleOverrideSchema
from .schedule_override_user_schema import ScheduleOverrideUserSchema
from .schedule_schema import ScheduleSchema
from .schedule_shift_schema import ScheduleShiftSchema
from .schedule_shift_user_schema import ScheduleShiftUserSchema
from .service_oncall_user_schema import ServiceOncallUserSchema
from .service_schema import ServiceSchema
from .update_heartbeat_input import UpdateHeartbeatInput
from .update_heartbeat_state_input import UpdateHeartbeatStateInput
from .user_schema import UserSchema

__all__ = (
    "CreateHeartbeatInput",
    "CreateHeartbeatStateInput",
    "CronHeartbeatStateSchema",
    "CronHeartbeatStateSchemaType",
    "HeartBeatFilter",
    "HeartbeatSchema",
    "HeartbeatType",
    "IncidentGroupDetailNoteSchema",
    "IncidentGroupDetailSchema",
    "IncidentGroupFilter",
    "IncidentGroupSchema",
    "IncidentGroupServiceSchema",
    "IncidentGroupStatus",
    "Input",
    "IntervalHeartbeatStateSchema",
    "IntervalHeartbeatStateSchemaType",
    "PagedHeartbeatSchema",
    "PagedIncidentGroupSchema",
    "PagedScheduleOverrideSchema",
    "PagedScheduleSchema",
    "PagedServiceSchema",
    "PagedUserSchema",
    "PageInfo",
    "ScheduleOverrideFilter",
    "ScheduleOverrideInputSchema",
    "ScheduleOverrideSchema",
    "ScheduleOverrideUserSchema",
    "ScheduleSchema",
    "ScheduleShiftSchema",
    "ScheduleShiftUserSchema",
    "ServiceOncallUserSchema",
    "ServiceSchema",
    "UpdateHeartbeatInput",
    "UpdateHeartbeatStateInput",
    "UserSchema",
)



from enum import Enum


class IncidentGroupStatus(str, Enum):
    ACKNOWLEDGED = "acknowledged"
    RESOLVED = "resolved"
    SILENCED = "silenced"
    TRIGGERED = "triggered"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define, field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Input")


@_attrs_define
class Input:
    """
    Attributes:
        page_size (Union[None, Unset, int]):
        before (Union[None, Unset, str]):
        after (Union[None, Unset, str]):
    """

    page_size: Union[None, Unset, int] = UNSET
    before: Union[None, Unset, str] = UNSET
    after: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        page_size: Union[None, Unset, int]
        if isinstance(self.page_size, Unset):
            page_size = UNSET
        else:
            page_size = self.page_size

        before: Union[None, Unset, str]
        if isinstance(self.before, Unset):
            before = UNSET
        else:
            before = self.before

        after: Union[None, Unset, str]
        if isinstance(self.after, Unset):
            after = UNSET
        else:
            after = self.after

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if page_size is not UNSET:
            field_dict["page_size"] = page_size
        if before is not UNSET:
            field_dict["before"] = before
        if after is not UNSET:
            field_dict["after"] = after

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_page_size(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        page_size = _parse_page_size(d.pop("page_size", UNSET))

        def _parse_before(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        before = _parse_before(d.pop("before", UNSET))

        def _parse_after(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        after = _parse_after(d.pop("after", UNSET))

        input_ = cls(
            page_size=page_size,
            before=before,
            after=after,
        )

        input_.additional_properties = d
        return input_

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define, field as _attrs_field

T = TypeVar("T", bound="PageInfo")


@_attrs_define
class PageInfo:
    """
    Attributes:
        has_next_page (bool):
        has_previous_page (bool):
        start_cursor (Union[None, str]):
        end_cursor (Union[None, str]):
    """

    has_next_page: bool
    has_previous_page: bool
    start_cursor: Union[None, str]
    end_cursor: Union[None, str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        has_next_page = self.has_next_page

        has_previous_page = self.has_previous_page

        start_cursor: Union[None, str]
        start_cursor = self.start_cursor

        end_cursor: Union[None, str]
        end_cursor = self.end_cursor

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "has_next_page": has_next_page,
                "has_previous_page": has_previous_page,
                "start_cursor": start_cursor,
                "end_cursor": end_cursor,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        has_next_page = d.pop("has_next_page")

        has_previous_page = d.pop("has_previous_page")

        def _parse_start_cursor(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        start_cursor = _parse_start_cursor(d.pop("start_cursor"))

        def _parse_end_cursor(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        end_cursor = _parse_end_cursor(d.pop("end_cursor"))

        page_info = cls(
            has_next_page=has_next_page,
            has_previous_page=has_previous_page,
            start_cursor=start_cursor,
            end_cursor=end_cursor,
        )

        page_info.additional_properties = d
        return page_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.paged_service_schema import PagedServiceSchema
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    page_size: Union[None, Unset, int] = UNSET,
    before: Union[None, Unset, str] = UNSET,
    after: Union[None, Unset, str] = UNSET,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    json_page_size: Union[None, Unset, int]
    if isinstance(page_size, Unset):
        json_page_size = UNSET
    else:
        json_page_size = page_size
    params["page_size"] = json_page_size

    json_before: Union[None, Unset, str]
    if isinstance(before, Unset):
        json_before = UNSET
    else:
        json_before = before
    params["before"] = json_before

    json_after: Union[None, Unset, str]
    if isinstance(after, Unset):
        json_after = UNSET
    else:
        json_after = after
    params["after"] = json_after

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/api/v1/services/",
        "params": params,
    }

    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[PagedServiceSchema]:
    if response.status_code == HTTPStatus.OK:
        response_200 = PagedServiceSchema.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[PagedServiceSchema]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    page_size: Union[None, Unset, int] = UNSET,
    before: Union[None, Unset, str] = UNSET,
    after: Union[None, Unset, str] = UNSET,
) -> Response[PagedServiceSchema]:
    """List Services

    Args:
        page_size (Union[None, Unset, int]):
        before (Union[None, Unset, str]):
        after (Union[None, Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PagedServiceSchema]
    """

    kwargs = _get_kwargs(
        page_size=page_size,
        before=before,
        after=after,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    page_size: Union[None, Unset, int] = UNSET,
    before: Union[None, Unset, str] = UNSET,
    after: Union[None, Unset, str] = UNSET,
) -> Optional[PagedServiceSchema]:
    """List Services

    Args:
        page_size (Union[None, Unset, int]):
        before (Union[None, Unset, str]):
        after (Union[None, Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PagedServiceSchema
    """

    return sync_detailed(
        client=client,
        page_size=page_size,
        before=before,
        after=after,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    page_size: Union[None, Unset, int] = UNSET,
    before: Union[None, Unset, str] = UNSET,
    after: Union[None, Unset, str] = UNSET,
) -> Response[PagedServiceSchema]:
    """List Services

    Args:
        page_size (Union[None, Unset, int]):
        before (Union[None, Unset, str]):
        after (Union[None, Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PagedServiceSchema]
    """

    kwargs = _get_kwargs(
        page_size=page_size,
        before=before,
        after=after,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    page_size: Union[None, Unset, int] = UNSET,
    before: Union[None, Unset, str] = UNSET,
    after: Union[None, Unset, str] = UNSET,
) -> Optional[PagedServiceSchema]:
    """List Services

    Args:
        page_size (Union[None, Unset, int]):
        before (Union[None, Unset, str]):
        after (Union[None, Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PagedServiceSchema
    """

    return (
        await asyncio_detailed(
            client=client,
            page_size=page_size,
            before=before,
            after=after,
        )
    ).parsed



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define, field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="ScheduleOverrideFilter")


@_attrs_define
class ScheduleOverrideFilter:
    """
    Attributes:
        since (Union[None, Unset, datetime.datetime]):
        until (Union[None, Unset, datetime.datetime]):
    """

    since: Union[None, Unset, datetime.datetime] = UNSET
    until: Union[None, Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        since: Union[None, Unset, str]
        if isinstance(self.since, Unset):
            since = UNSET
        elif isinstance(self.since, datetime.datetime):
            since = self.since.isoformat()
        else:
            since = self.since

        until: Union[None, Unset, str]
        if isinstance(self.until, Unset):
            until = UNSET
        elif isinstance(self.until, datetime.datetime):
            until = self.until.isoformat()
        else:
            until = self.until

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if since is not UNSET:
            field_dict["since"] = since
        if until is not UNSET:
            field_dict["until"] = until

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_since(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                since_type_0 = isoparse(data)

                return since_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        since = _parse_since(d.pop("since", UNSET))

        def _parse_until(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                until_type_0 = isoparse(data)

                return until_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        until = _parse_until(d.pop("until", UNSET))

        schedule_override_filter = cls(
            since=since,
            until=until,
        )

        schedule_override_filter.additional_properties = d
        return schedule_override_filter

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define, field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="HeartBeatFilter")


@_attrs_define
class HeartBeatFilter:
    """
    Attributes:
        service (Union[None, Unset, int]):
    """

    service: Union[None, Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        service: Union[None, Unset, int]
        if isinstance(self.service, Unset):
            service = UNSET
        else:
            service = self.service

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if service is not UNSET:
            field_dict["service"] = service

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_service(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        service = _parse_service(d.pop("service", UNSET))

        heart_beat_filter = cls(
            service=service,
        )

        heart_beat_filter.additional_properties = d
        return heart_beat_filter

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define, field as _attrs_field

from ..models.cron_heartbeat_state_schema_type import CronHeartbeatStateSchemaType
from ..types import UNSET, Unset

T = TypeVar("T", bound="CronHeartbeatStateSchema")


@_attrs_define
class CronHeartbeatStateSchema:
    """
    Attributes:
        type (CronHeartbeatStateSchemaType):
        cron_expression (str):
        cron_timezone (str):
        id (Union[None, Unset, int]):
        environment (Union[None, Unset, str]):
        muted (Union[Unset, bool]):  Default: False.
        resolve_incident (Union[Unset, bool]):  Default: True.
        timeout_seconds (Union[Unset, int]):  Default: 0.
    """

    type: CronHeartbeatStateSchemaType
    cron_expression: str
    cron_timezone: str
    id: Union[None, Unset, int] = UNSET
    environment: Union[None, Unset, str] = UNSET
    muted: Union[Unset, bool] = False
    resolve_incident: Union[Unset, bool] = True
    timeout_seconds: Union[Unset, int] = 0
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        cron_expression = self.cron_expression

        cron_timezone = self.cron_timezone

        id: Union[None, Unset, int]
        if isinstance(self.id, Unset):
            id = UNSET
        else:
            id = self.id

        environment: Union[None, Unset, str]
        if isinstance(self.environment, Unset):
            environment = UNSET
        else:
            environment = self.environment

        muted = self.muted

        resolve_incident = self.resolve_incident

        timeout_seconds = self.timeout_seconds

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "cron_expression": cron_expression,
                "cron_timezone": cron_timezone,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if environment is not UNSET:
            field_dict["environment"] = environment
        if muted is not UNSET:
            field_dict["muted"] = muted
        if resolve_incident is not UNSET:
            field_dict["resolve_incident"] = resolve_incident
        if timeout_seconds is not UNSET:
            field_dict["timeout_seconds"] = timeout_seconds

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = CronHeartbeatStateSchemaType(d.pop("type"))

        cron_expression = d.pop("cron_expression")

        cron_timezone = d.pop("cron_timezone")

        def _parse_id(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        id = _parse_id(d.pop("id", UNSET))

        def _parse_environment(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        environment = _parse_environment(d.pop("environment", UNSET))

        muted = d.pop("muted", UNSET)

        resolve_incident = d.pop("resolve_incident", UNSET)

        timeout_seconds = d.pop("timeout_seconds", UNSET)

        cron_heartbeat_state_schema = cls(
            type=type,
            cron_expression=cron_expression,
            cron_timezone=cron_timezone,
            id=id,
            environment=environment,
            muted=muted,
            resolve_incident=resolve_incident,
            timeout_seconds=timeout_seconds,
        )

        cron_heartbeat_state_schema.additional_properties = d
        return cron_heartbeat_state_schema

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
